@@ -6,6 +6,12 @@ from __future__ import absolute_import
 
 import copy
 import re
+
+try:
+    import __builtin__ as builtins
+except ImportError:  # Py3
+    import builtins
+
 from .Errors import warning, error, InternalError
 from .StringEncoding import EncodedString
 from . import Options, Naming
@@ -14,8 +20,8 @@ from .PyrexTypes import py_object_type, unspecified_type
 from .TypeSlots import \
     pyfunction_signature, pymethod_signature, \
     get_special_method_signature, get_property_accessor_signature
+
 from . import Code
-import __builtin__ as builtins
 
 iso_c99_keywords = set(
 ['auto', 'break', 'case', 'char', 'const', 'continue', 'default', 'do',
@@ -338,7 +344,7 @@ class Scope(object):
     def merge_in(self, other, merge_unused=True, whitelist=None):
         # Use with care...
         entries = []
-        for name, entry in other.entries.iteritems():
+        for name, entry in other.entries.items():
             if not whitelist or name in whitelist:
                 if entry.used or merge_unused:
                     entries.append((name, entry))
@@ -490,7 +496,7 @@ class Scope(object):
         try:
             type = PyrexTypes.create_typedef_type(name, base_type, cname,
                                                   (visibility == 'extern'))
-        except ValueError, e:
+        except ValueError as e:
             error(pos, e.args[0])
             type = PyrexTypes.error_type
         entry = self.declare_type(name, type, pos, cname,
@@ -698,8 +704,15 @@ class Scope(object):
                 cname = self.mangle(Naming.func_prefix, name)
         entry = self.lookup_here(name)
         if entry:
+            if not in_pxd and visibility != entry.visibility and visibility == 'extern':
+                # Previously declared, but now extern => treat this
+                # as implementing the function, using the new cname
+                defining = True
+                visibility = entry.visibility
+                entry.cname = cname
+                entry.func_cname = cname
             if visibility != 'private' and visibility != entry.visibility:
-                warning(pos, "Function '%s' previously declared as '%s'" % (name, entry.visibility), 1)
+                warning(pos, "Function '%s' previously declared as '%s', now as '%s'" % (name, entry.visibility, visibility), 1)
             if overridable != entry.is_overridable:
                 warning(pos, "Function '%s' previously declared as '%s'" % (
                     name, 'cpdef' if overridable else 'cdef'), 1)
@@ -886,7 +899,7 @@ class BuiltinScope(Scope):
             Scope.__init__(self, "__builtin__", PreImportScope(), None)
         self.type_names = {}
 
-        for name, definition in self.builtin_entries.iteritems():
+        for name, definition in sorted(self.builtin_entries.items()):
             cname, type = definition
             self.declare_var(name, type, None, cname)
 
@@ -941,7 +954,7 @@ class BuiltinScope(Scope):
         var_entry = Entry(name = entry.name,
             type = self.lookup('type').type, # make sure "type" is the first type declared...
             pos = entry.pos,
-            cname = "((PyObject*)%s)" % entry.type.typeptr_cname)
+            cname = entry.type.typeptr_cname)
         var_entry.is_variable = 1
         var_entry.is_cglobal = 1
         var_entry.is_readonly = 1
@@ -1127,28 +1140,49 @@ class ModuleScope(Scope):
         # relative imports relative to this module's parent.
         # Finds and parses the module's .pxd file if the module
         # has not been referenced before.
-        module_scope = self.global_scope()
+        relative_to = None
+        absolute_fallback = False
         if relative_level is not None and relative_level > 0:
-            # merge current absolute module name and relative import name into qualified name
-            current_module = module_scope.qualified_name.split('.')
-            base_package = current_module[:-relative_level]
-            module_name = '.'.join(base_package + (module_name.split('.') if module_name else []))
+            # explicit relative cimport
+            # error of going beyond top-level is handled in cimport node
+            relative_to = self
+            while relative_level > 0 and relative_to:
+                relative_to = relative_to.parent_module
+                relative_level -= 1
+        elif relative_level != 0:
+            # -1 or None: try relative cimport first, then absolute
+            relative_to = self.parent_module
+            absolute_fallback = True
+
+        module_scope = self.global_scope()
         return module_scope.context.find_module(
-            module_name, relative_to=None if relative_level == 0 else self.parent_module, pos=pos)
+            module_name, relative_to=relative_to, pos=pos, absolute_fallback=absolute_fallback)
 
     def find_submodule(self, name):
         # Find and return scope for a submodule of this module,
         # creating a new empty one if necessary. Doesn't parse .pxd.
+        if '.' in name:
+            name, submodule = name.split('.', 1)
+        else:
+            submodule = None
         scope = self.lookup_submodule(name)
         if not scope:
-            scope = ModuleScope(name,
-                parent_module = self, context = self.context)
+            scope = ModuleScope(name, parent_module=self, context=self.context)
             self.module_entries[name] = scope
+        if submodule:
+            scope = scope.find_submodule(submodule)
         return scope
 
     def lookup_submodule(self, name):
         # Return scope for submodule of this module, or None.
-        return self.module_entries.get(name, None)
+        if '.' in name:
+            name, submodule = name.split('.', 1)
+        else:
+            submodule = None
+        module = self.module_entries.get(name, None)
+        if submodule and module is not None:
+            module = module.lookup_submodule(submodule)
+        return module
 
     def add_include_file(self, filename):
         if filename not in self.python_include_files \
@@ -1509,7 +1543,7 @@ class ModuleScope(Scope):
         var_entry = Entry(name = entry.name,
             type = Builtin.type_type,
             pos = entry.pos,
-            cname = "((PyObject*)%s)" % entry.type.typeptr_cname)
+            cname = entry.type.typeptr_cname)
         var_entry.is_variable = 1
         var_entry.is_cglobal = 1
         var_entry.is_readonly = 1
